!ORILAM_LIC Copyright 1994-2014 CNRS, Meteo-France and Universite Paul Sabatier
!ORILAM_LIC This is part of the ORILAM software governed by the CeCILL-C licence
!ORILAM_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!ORILAM_LIC for details.
MODULE MODD_DST_n

!Purpose: 
!Declare variables and constants necessary to do the dust calculations
!Here are only the variables which depend on the grid!

!Author: Alf Grini <alf.grini@cnrm.meteo.fr>

!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE DST_t
  !
  !
  INTEGER, DIMENSION(:), POINTER     :: NVT_DST=>NULL()           !MASK: dust vegetation number to vegetation number
  INTEGER, DIMENSION(:,:), POINTER   :: NSIZE_PATCH_DST=>NULL()   !Number of points for a patch and a vegetation class
  INTEGER, DIMENSION(:,:,:), POINTER :: NR_PATCH_DST=>NULL()      !Mask from patch-points to dust-points
  REAL,DIMENSION(:), POINTER         :: Z0_EROD_DST=>NULL()       !Roughness length momentum over erodible dust emitter sfc
  CHARACTER(LEN=6), DIMENSION(:), POINTER  :: CSV_DST =>NULL()          !Name of scalar variables 
  REAL, DIMENSION(:,:,:),POINTER    :: XSFDST=>NULL()            !Dust variables to be send to output
  REAL, DIMENSION(:,:,:),POINTER    :: XSFDSTM=>NULL()            !Dust variables to be send to output
  REAL,DIMENSION(:), POINTER   :: XEMISRADIUS_DST =>NULL()               !Number median radius for each source mode
  REAL,DIMENSION(:), POINTER   :: XEMISSIG_DST  =>NULL()                 !sigma for each source mode
  REAL,DIMENSION(:), POINTER   :: XMSS_FRC_SRC =>NULL()              !Mass fraction of each source mode
END TYPE DST_t

TYPE(DST_t), ALLOCATABLE, TARGET, SAVE :: DST_MODEL(:)

INTEGER, DIMENSION(:), POINTER     :: NVT_DST=>NULL()
!$OMP THREADPRIVATE(NVT_DST)
INTEGER, DIMENSION(:,:), POINTER   :: NSIZE_PATCH_DST=>NULL()
!$OMP THREADPRIVATE(NSIZE_PATCH_DST)
INTEGER, DIMENSION(:,:,:), POINTER :: NR_PATCH_DST=>NULL()
!$OMP THREADPRIVATE(NR_PATCH_DST)
REAL,DIMENSION(:), POINTER         :: Z0_EROD_DST=>NULL()
!$OMP THREADPRIVATE(Z0_EROD_DST)
 CHARACTER(LEN=6), DIMENSION(:), POINTER   :: CSV_DST=>NULL()
!$OMP THREADPRIVATE(CSV_DST)
REAL, DIMENSION(:,:,:),POINTER     :: XSFDST=>NULL()
!$OMP THREADPRIVATE(XSFDST)
REAL, DIMENSION(:,:,:),POINTER     :: XSFDSTM=>NULL()
!$OMP THREADPRIVATE(XSFDSTM)
REAL,DIMENSION(:), POINTER   :: XEMISRADIUS_DST=>NULL()
!$OMP THREADPRIVATE(XEMISRADIUS_DST)
REAL,DIMENSION(:), POINTER   :: XEMISSIG_DST=>NULL()
!$OMP THREADPRIVATE(XEMISSIG_DST)
REAL,DIMENSION(:), POINTER   :: XMSS_FRC_SRC=>NULL()
!$OMP THREADPRIVATE(XMSS_FRC_SRC)

CONTAINS

SUBROUTINE DST_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
DST_MODEL(KFROM)%NVT_DST=>NVT_DST
DST_MODEL(KFROM)%NSIZE_PATCH_DST=>NSIZE_PATCH_DST
DST_MODEL(KFROM)%NR_PATCH_DST=>NR_PATCH_DST
DST_MODEL(KFROM)%Z0_EROD_DST=>Z0_EROD_DST
DST_MODEL(KFROM)%CSV_DST=>CSV_DST
DST_MODEL(KFROM)%XSFDST=>XSFDST
DST_MODEL(KFROM)%XSFDSTM=>XSFDSTM
DST_MODEL(KFROM)%XEMISRADIUS_DST=>XEMISRADIUS_DST
DST_MODEL(KFROM)%XEMISSIG_DST=>XEMISSIG_DST
DST_MODEL(KFROM)%XMSS_FRC_SRC=>XMSS_FRC_SRC
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_DST_N:DST_GOTO_MODEL',0,ZHOOK_HANDLE)
NVT_DST=>DST_MODEL(KTO)%NVT_DST
NSIZE_PATCH_DST=>DST_MODEL(KTO)%NSIZE_PATCH_DST
NR_PATCH_DST=>DST_MODEL(KTO)%NR_PATCH_DST
Z0_EROD_DST=>DST_MODEL(KTO)%Z0_EROD_DST
CSV_DST=>DST_MODEL(KTO)%CSV_DST
XSFDST=>DST_MODEL(KTO)%XSFDST
XSFDSTM=>DST_MODEL(KTO)%XSFDSTM
XEMISRADIUS_DST=>DST_MODEL(KTO)%XEMISRADIUS_DST
XEMISSIG_DST=>DST_MODEL(KTO)%XEMISSIG_DST
XMSS_FRC_SRC=>DST_MODEL(KTO)%XMSS_FRC_SRC
IF (LHOOK) CALL DR_HOOK('MODD_DST_N:DST_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE DST_GOTO_MODEL

SUBROUTINE DST_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DST_N:DST_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(DST_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(DST_MODEL(J)%NVT_DST)
  NULLIFY(DST_MODEL(J)%NSIZE_PATCH_DST)
  NULLIFY(DST_MODEL(J)%NR_PATCH_DST)
  NULLIFY(DST_MODEL(J)%Z0_EROD_DST)
  NULLIFY(DST_MODEL(J)%CSV_DST)
  NULLIFY(DST_MODEL(J)%XSFDST)
  NULLIFY(DST_MODEL(J)%XSFDSTM)
  NULLIFY(DST_MODEL(J)%XEMISRADIUS_DST)
  NULLIFY(DST_MODEL(J)%XEMISSIG_DST)
  NULLIFY(DST_MODEL(J)%XMSS_FRC_SRC)
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_DST_N:DST_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE DST_ALLOC

SUBROUTINE DST_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_DST_N:DST_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(DST_MODEL)) DEALLOCATE(DST_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_DST_N:DST_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE DST_DEALLO

END MODULE MODD_DST_n
