!SURFEX_LIC Copyright 1994-2014 Meteo-France 
!SURFEX_LIC This is part of the SURFEX software governed by the CeCILL-C  licence
!SURFEX_LIC version 1. See LICENSE, CeCILL-C_V1-en.txt and CeCILL-C_V1-fr.txt
!SURFEX_LIC for details. version 1.
MODULE MODD_SLT_n

!Purpose: 
!Declare variables and constants necessary to do the sea salt calculations
!Here are only the variables which depend on the grid!
!
!Author: Alf Grini / Pierre Tulet
!
USE YOMHOOK   ,ONLY : LHOOK,   DR_HOOK
USE PARKIND1  ,ONLY : JPRB
!
IMPLICIT NONE

TYPE SLT_t
  REAL, DIMENSION(:,:,:),POINTER :: XSFSLT                      ! Sea Salt variables to be send to output
  REAL,DIMENSION(:), POINTER     :: XEMISRADIUS_SLT             ! Number median radius for each source mode
  REAL,DIMENSION(:), POINTER     :: XEMISSIG_SLT                ! sigma for each source mode
END TYPE SLT_t

TYPE(SLT_t), ALLOCATABLE, TARGET, SAVE :: SLT_MODEL(:)

REAL, DIMENSION(:,:,:),POINTER  :: XSFSLT=>NULL()
!$OMP THREADPRIVATE(XSFSLT)
REAL,DIMENSION(:), POINTER      :: XEMISRADIUS_SLT=>NULL()
!$OMP THREADPRIVATE(XEMISRADIUS_SLT)
REAL,DIMENSION(:), POINTER      :: XEMISSIG_SLT=>NULL()
!$OMP THREADPRIVATE(XEMISSIG_SLT)

CONTAINS

SUBROUTINE SLT_GOTO_MODEL(KFROM, KTO, LKFROM)
LOGICAL, INTENT(IN) :: LKFROM
INTEGER, INTENT(IN) :: KFROM, KTO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
!
! Save current state for allocated arrays
IF (LKFROM) THEN
SLT_MODEL(KFROM)%XSFSLT=>XSFSLT
SLT_MODEL(KFROM)%XEMISRADIUS_SLT=>XEMISRADIUS_SLT
SLT_MODEL(KFROM)%XEMISSIG_SLT=>XEMISSIG_SLT
ENDIF
!
! Current model is set to model KTO
IF (LHOOK) CALL DR_HOOK('MODD_SLT_N:SLT_GOTO_MODEL',0,ZHOOK_HANDLE)
XSFSLT=>SLT_MODEL(KTO)%XSFSLT
XEMISRADIUS_SLT=>SLT_MODEL(KTO)%XEMISRADIUS_SLT
XEMISSIG_SLT=>SLT_MODEL(KTO)%XEMISSIG_SLT
IF (LHOOK) CALL DR_HOOK('MODD_SLT_N:SLT_GOTO_MODEL',1,ZHOOK_HANDLE)

END SUBROUTINE SLT_GOTO_MODEL

SUBROUTINE SLT_ALLOC(KMODEL)
INTEGER, INTENT(IN) :: KMODEL
INTEGER :: J
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SLT_N:SLT_ALLOC",0,ZHOOK_HANDLE)
ALLOCATE(SLT_MODEL(KMODEL))
DO J=1,KMODEL
  NULLIFY(SLT_MODEL(J)%XSFSLT)
  NULLIFY(SLT_MODEL(J)%XEMISRADIUS_SLT)
  NULLIFY(SLT_MODEL(J)%XEMISSIG_SLT)
ENDDO
IF (LHOOK) CALL DR_HOOK("MODD_SLT_N:SLT_ALLOC",1,ZHOOK_HANDLE)
END SUBROUTINE SLT_ALLOC

SUBROUTINE SLT_DEALLO
REAL(KIND=JPRB) :: ZHOOK_HANDLE
IF (LHOOK) CALL DR_HOOK("MODD_SLT_N:SLT_DEALLO",0,ZHOOK_HANDLE)
IF (ALLOCATED(SLT_MODEL)) DEALLOCATE(SLT_MODEL)
IF (LHOOK) CALL DR_HOOK("MODD_SLT_N:SLT_DEALLO",1,ZHOOK_HANDLE)
END SUBROUTINE SLT_DEALLO

END MODULE MODD_SLT_n
